use categorized_hand::CategorizedHand;

mod categorized_hand;
mod dealt_hand;
mod hand_kind;
mod parsing;
mod second_ordering;

pub fn solve_task_1(input: &str) -> String {
    let parsed = parsing::parse_input(input);
    let sorted_and_categorized = {
        let mut categorized: Vec<_> = parsed.into_iter().map(CategorizedHand::from).collect();
        categorized.sort();
        categorized
    };
    let total_winning = calc_total_winning(&sorted_and_categorized);
    total_winning.to_string()
}

fn calc_total_winning(sorted: &[CategorizedHand]) -> usize {
    sorted
        .into_iter()
        .enumerate()
        .map(|(index, hand)| {
            let rank = index.saturating_add(1);
            let score = hand.bid();
            score.checked_mul(rank).unwrap_or_else(|| {
                panic!(
                    "Multiplication of score ({}) with rank ({}) resulted in overflow",
                    score, rank
                )
            })
        })
        .sum()
}

#[cfg(test)]
const TEST_INPUT: &str = "32T3K 765
T55J5 684
KK677 28
KTJJT 220
QQQJA 483
";

#[cfg(test)]
mod testing {
    use super::TEST_INPUT;

    #[test]
    fn solve_example_according_to_task_1() {
        let actual = super::solve_task_1(TEST_INPUT);
        let expected = "6440";
        assert_eq!(expected, actual);
    }
}
