use std::{cmp::Ordering, collections::HashMap, ops::RangeInclusive, sync::LazyLock, usize};

use log::warn;

static LETTERS: &[char] = &['T', 'J', 'Q', 'K', 'A'];
const NUMBERS: RangeInclusive<u32> = 0..=9;

static LABEL_ORDERING: LazyLock<HashMap<char, usize>> = LazyLock::new(|| {
    let numbers = NUMBERS
        .into_iter()
        .map(|number| std::char::from_digit(number, 10).unwrap());
    numbers
        .chain(LETTERS.into_iter().copied())
        .into_iter()
        .enumerate()
        .map(|(index, character)| (character, index))
        .collect()
});
//
//A hand consists of five cards labeled one of A, K, Q, J, T, 9, 8, 7, 6, 5, 4, 3, or 2.
//The relative strength of each card follows this order, where A is the highest and 2 is the lowest.
pub fn compare_along_str(left: &str, right: &str) -> Ordering {
    for (left_char, right_char) in left.chars().zip(right.chars()) {
        let left_rank_value = LABEL_ORDERING
            .get(&left_char)
            .unwrap_or_else(|| panic!("left char ({}) is not valid", left_char));
        let right_rank_value = LABEL_ORDERING
            .get(&right_char)
            .unwrap_or_else(|| panic!("right char ({}) is not valid", right_char));

        let current_ordering = left_rank_value.cmp(&right_rank_value);

        let found_difference = current_ordering != Ordering::Equal;
        if found_difference {
            return current_ordering;
        }
    }
    let (left_len, right_len) = (left.len(), right.len());
    warn!(
        "Left and right seem to have no difference in strength\n\
        Now comparing via lenght ({}) ({}). By same length, lefte wins !",
        left_len, right_len
    );
    left_len.cmp(&right_len)
}
